#property strict

#define MAX_BUFFERS 7

#property indicator_separate_window_window
#property indicator_buffers MAX_BUFFERS

sinput int MaxCalcBars = PERIOD_D1 * 10; // Max Calculate Bars

class INDICATOR
{
private:
  struct BUFFER
  {
    double Buffer[];
  };

  void Init( void )
  {
    ArrayResize(Buffers, Size);

    for (int i = 0; i < Size; i++)
      SetIndexBuffer(i, Buffers[i].Buffer);

    return;
  }

protected:
  BUFFER Buffers[];
  int Size;

public:

  INDICATOR( const int &Colors[] )
  {
    Size = MathMin(ArraySize(Colors), MAX_BUFFERS);

    Init();
  }

  void BuffersSetAsSeries( const bool Flag )
  {
    for (int i = 0; i < Size; i++)
      ArraySetAsSeries(Buffers[i].Buffer, Flag);

    return;
  }


  virtual int Calculate(const int rates_total,
                        const int prev_calculated,
                        const datetime &time[],
                        const double &open[],
                        const double &high[],
                        const double &low[],
                        const double &close[],
                        const long &tick_volume[],
                        const long &volume[],
                        const int &spread[])
  {
    return(rates_total);
  }
};

/*--------------------------------------------------------*/

input int period = PERIOD_H1; // Period in bars

#define EMPTY_CURRENCY ""
#define MAX_DIGITS 8

class RELATION : INDICATOR
{
private:
  struct SYMBOL
  {
    string Name;
    bool Direction;

    void SetSymbol( const string Symb, const string BaseCurrency )
    {
      Name = Symb;

      Direction = (StringSubstr(Symb, 0, 3) == BaseCurrency);

      return;
    }

    double GetLogPrice( const datetime time )
    {
      return(MathLog(iClose(Name, Period(), iBarShift(Name, Period(), time))));
    }

    double GetRelation( const datetime TimeLeft, const datetime TimeRight )
    {
      double Res = GetLogPrice(TimeRight) - GetLogPrice(TimeLeft);

      if (Direction)
        Res = -Res;

      return(Res);
    }
  };

  SYMBOL Symbols[];

  string GetBaseCurrency( const string Symbol1, const string Symbol2 )
  {
    string Currency = StringSubstr(Symbol1, 0, 3);

    if (StringFind(Symbol2, Currency) >= 0)
      return(Currency);

    Currency = StringSubstr(Symbol1, 3, 3);

    if (StringFind(Symbol2, Currency) >= 0)
      return(Currency);

    return(EMPTY_CURRENCY);
  }

public:
  RELATION( const string &SymbolList[], const int &Colors[] ) : INDICATOR(Colors)
  {
    string Currency1 = StringSubstr(Symbol(), 0, 3);
    string Currency2 = StringSubstr(Symbol(), 3, 3);

    string BaseCurrency = EMPTY_CURRENCY;

    if (Size > ArraySize(SymbolList))
    {
      Size = ArraySize(SymbolList);

      ArrayResize(Buffers, Size);
    }

    ArrayResize(Symbols, Size);

    if (Size > 1)
      BaseCurrency = GetBaseCurrency(SymbolList[0], SymbolList[1]);

    if (Currency1 == BaseCurrency)
      Currency1 = EMPTY_CURRENCY;
    else if (Currency2 == BaseCurrency)
      Currency2 = EMPTY_CURRENCY;

    IndicatorDigits(MAX_DIGITS);

    for (int i = 0; i < Size; i++)
    {
      if ((StringFind(SymbolList[i], Currency1) >= 0) || (StringFind(SymbolList[i], Currency2) >= 0))
        SetIndexStyle(i, DRAW_LINE, EMPTY, 2, Colors[i]);
      else
        SetIndexStyle(i, DRAW_LINE, EMPTY, 1, Colors[i]);

      SetIndexLabel(i, SymbolList[i]);

      Symbols[i].SetSymbol(SymbolList[i], BaseCurrency);
    }

    BuffersSetAsSeries(FALSE);

    return;
  }

  virtual int Calculate(const int rates_total,
                        const int prev_calculated,
                        const datetime &time[],
                        const double &open[],
                        const double &high[],
                        const double &low[],
                        const double &close[],
                        const long &tick_volume[],
                        const long &volume[],
                        const int &spread[])
  {
    ArraySetAsSeries(time, FALSE);

    const int Begin = MathMax(MathMax(prev_calculated - 1, rates_total - MaxCalcBars), period);
    const int SymbolsSize = ArraySize(Symbols);

    for (int i = Begin; i < rates_total; i++)
      for (int j = 0; j < SymbolsSize; j++)
        Buffers[j].Buffer[i] = Symbols[j].GetRelation(time[i - period], time[i]);

    return(rates_total);
  }
};

/*--------------------------------------------------------*/

INDICATOR* Relation;

void OnInit( void )
{
  const int Colors[] = {clrRed, clrBlue, clrSandyBrown, clrThistle, clrLime, clrSandyBrown, clrMagenta};
  const string SymbolList[] = {"EURUSD", "GBPUSD", "AUDUSD", "NZDUSD", "USDJPY", "USDCHF", "USDCAD"};

  Relation = new RELATION(SymbolList, Colors);

  return;
}

void OnDeinit( const int reason )
{
  delete Relation;

  return;
}

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
  return(Relation.Calculate(rates_total, prev_calculated, time, open, high, low, close, tick_volume, volume, spread));
}